package es.kauron.jstudy.controller;

import es.kauron.jstudy.Main;
import es.kauron.jstudy.model.AppConfig;
import es.kauron.jstudy.model.TestItem;
import javafx.application.Platform;
import javafx.beans.property.BooleanProperty;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.event.ActionEvent;
import javafx.event.Event;
import javafx.event.EventHandler;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.fxml.Initializable;
import javafx.scene.Parent;
import javafx.scene.control.*;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.layout.BorderPane;
import javafx.stage.FileChooser;

import java.awt.*;
import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.net.URL;
import java.util.*;
import java.util.List;

public class Controller implements Initializable {
    @FXML
    private TabPane tabPane;

    @FXML
    private BorderPane root;

    private BooleanProperty table = new SimpleBooleanProperty(false);
    private Map<Tab, TableController> tabMap = new HashMap<>();

    @Override
    public void initialize(URL url, ResourceBundle resourceBundle) {
        tabPane.getSelectionModel().selectedItemProperty().addListener((ob, o, n) -> table.set(tabMap.get(n) != null));
        Platform.runLater(() ->
            root.getScene().getWindow().setOnCloseRequest(event -> {
                for (Tab tab : tabPane.getTabs()) {
                    EventHandler<Event> handler = tab.getOnCloseRequest();
                    if (tab.isClosable() && handler != null) {
                        tabPane.getSelectionModel().select(tab);
                        handler.handle(event);
                        if (event.isConsumed()) return;
                    }
                }
            })
        );
    }

    @FXML
    private void onSettingsAction(ActionEvent event) {
        try {
            Parent root = FXMLLoader.load(Main.class.getResource("view/settings.fxml"));
            tabPane.getTabs().add(new Tab("Settings", root));
            tabPane.getSelectionModel().selectLast();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @FXML
    private void onNewAction(ActionEvent event) {
        TextInputDialog dialog = new TextInputDialog();
        dialog.getEditor().setPromptText("Table name");
        dialog.setTitle("Creating new table");
        dialog.setHeaderText("Please input a name for the new table");
        dialog.setGraphic(new ImageView(new Image(Main.class.getResource("img/Edit.png").toString())));
        dialog.getDialogPane().setMinWidth(250);
        dialog.showAndWait();
        dialog.setResultConverter(value -> value.getButtonData().equals(ButtonBar.ButtonData.OK_DONE) ? value.getText() : "");
        if (dialog.getResult() == null || dialog.getResult().isEmpty()) return;
        tabPane.getTabs().add(createTableTab(dialog.getResult(), new ArrayList<>(), null));
        tabPane.getSelectionModel().selectLast();
    }

    @FXML
    private void onLoadAction(ActionEvent event) {
        FileChooser chooser = new FileChooser();
        chooser.getExtensionFilters().add(new FileChooser.ExtensionFilter("JStudy file", "*.jsdb"));
        if (AppConfig.lastDir != null) chooser.setInitialDirectory(AppConfig.lastDir);
        List<File> list = chooser.showOpenMultipleDialog(root.getScene().getWindow());
        if (list == null || list.isEmpty()) return;
        AppConfig.lastDir = list.get(0).getParentFile();
        if (list.size() > 1) {
            ButtonType mergeBT = new ButtonType("Merge", ButtonBar.ButtonData.APPLY);
            ButtonType openBT  = new ButtonType("Open",  ButtonBar.ButtonData.OK_DONE);
            Alert alert = new Alert(Alert.AlertType.CONFIRMATION);
            alert.getDialogPane().getButtonTypes().clear();
            alert.getDialogPane().getButtonTypes().addAll(mergeBT, openBT, ButtonType.CANCEL);
            alert.setTitle("Opening files...");
            alert.setHeaderText("Several files have been opened");
            alert.setContentText("Merge files or open them separately?");

            Optional<ButtonType> result = alert.showAndWait();

            if (result.isPresent() && result.get().equals(mergeBT)) {
                List<TestItem> aux = new ArrayList<>();
                for (int i = 0; i < list.size(); i++) {
                    aux.addAll(TestItem.loadFrom(list.get(i), TestItem.COLONS));
                }
                tabPane.getTabs().add(createTableTab("Merge table", aux, null));
                tabPane.getSelectionModel().selectLast();
            } else if (result.isPresent() && result.get().equals(openBT)){
                for (int i = 0; i < list.size(); i++) {
                    File file = list.get(i);
                    List<TestItem> aux = TestItem.loadFrom(file, TestItem.COLONS);
                    if (aux != null) {
                        tabPane.getTabs().add(createTableTab(file.getName().substring(0, file.getName().lastIndexOf('.')), aux, file));
                    }
                }
            }
        } else {
            File file = list.get(0);
            List<TestItem> aux = TestItem.loadFrom(file, TestItem.COLONS);
            if (aux != null) {
                tabPane.getTabs().add(createTableTab(file.getName().substring(0, file.getName().lastIndexOf('.')), aux, file));
                tabPane.getSelectionModel().selectLast();
            }
        }
    }

    @FXML
    private void onImportAction(ActionEvent event) {
        FileChooser chooser = new FileChooser();
        chooser.getExtensionFilters().add(new FileChooser.ExtensionFilter("Data files (.csv, .txt)", "*.csv", "*.txt"));
        if (AppConfig.lastDir != null) chooser.setInitialDirectory(AppConfig.lastDir);
        File file = chooser.showOpenDialog(root.getScene().getWindow());
        if (file == null) return;
        AppConfig.lastDir = file.getParentFile();
        String separator;
        if (file.getName().matches(".*txt"))
            separator = TestItem.TAB;
        else if (file.getName().matches(".*csv"))
            separator = TestItem.SEMICOLON;
        else
            separator = TestItem.COMMA;
        List<TestItem> aux = TestItem.loadFrom(file, separator);
        if (aux != null) {
            tabPane.getTabs().add(createTableTab(file.getName().substring(0, file.getName().lastIndexOf('.')), aux, null));
            tabPane.getSelectionModel().selectLast();
        }
    }

    private Tab createTableTab(String name, List<TestItem> list, File file) {
        try {
            Tab tab = new Tab(name);

            FXMLLoader loader = new FXMLLoader(Main.class.getResource("view/table.fxml"));
            Parent tableRoot = loader.load();

            ((TableController) loader.getController()).setData(name, list, this, file);
            tabMap.put(tab, loader.getController());

            tab.setContent(tableRoot);
            tab.setOnCloseRequest(event -> {
                if (!((TableController) loader.getController()).saved.get()) {
                    Alert dialog = new Alert(Alert.AlertType.WARNING);
                    dialog.setHeaderText("The tab " + name + " has unsaved information");
                    dialog.setContentText("Do you want to save those changes?");
                    dialog.getButtonTypes().clear();
                    dialog.getButtonTypes().addAll(ButtonType.YES, ButtonType.NO, ButtonType.CANCEL);
                    dialog.showAndWait();
                    if (dialog.getResult().equals(ButtonType.YES)) {
                        ((TableController) loader.getController()).onSaveAction(null);
                    } else if (dialog.getResult().equals(ButtonType.CANCEL)) {
                        event.consume();
                    }
                }
            });
            return tab;
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }

    void newTest(List<TestItem> list) {
        try {
            FXMLLoader loader = new FXMLLoader(Main.class.getResource("view/test.fxml"));
            Parent root = loader.load();

            ((TestController) loader.getController()).setList(new ArrayList<>(list));

            tabPane.getTabs().add(new Tab("Test: " + tabPane.getSelectionModel().getSelectedItem().getText(), root));
            tabPane.getSelectionModel().selectLast();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @FXML
    protected void onAboutAction(ActionEvent event) {
        if (Desktop.isDesktopSupported()) {
            try {
                Desktop.getDesktop().browse(URI.create("https://kauron.github.io/jstudy"));
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }
}
